##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""XXX short summary goes here.

$Id: tests.py 40673 2005-12-09 22:09:28Z jim $
"""
import unittest
from zope.testing import doctest

def type_errors():
    """
You can pass unicode strings, or strings:

    >>> from zope.ucol import Collator
    >>> c = Collator('root')
    >>> c.key(u"Hello") == c.key("Hello")
    True
    >>> c.cmp(u"Hello", "Hello")
    0

As long as the strings can be decoded as ASCII:

    >>> c.key("Hello\xfa")
    Traceback (most recent call last):
    ...
    UnicodeDecodeError: 'ascii' codec can't decode byte
    0xfa in position 5: ordinal not in range(128)

    >>> c.cmp(u"Hello", "Hello\xfa")
    Traceback (most recent call last):
    ...
    UnicodeDecodeError: 'ascii' codec can't decode byte
    0xfa in position 5: ordinal not in range(128)

And you can't pass a non-string:

    >>> c.key(0)
    Traceback (most recent call last):
    ...
    TypeError: Expected unicode string

    >>> c.cmp(u"Hello", 0)
    Traceback (most recent call last):
    ...
    TypeError: Expected unicode string

"""

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite(optionflags=doctest.NORMALIZE_WHITESPACE),
        doctest.DocFileSuite('README.txt',
                             optionflags=doctest.NORMALIZE_WHITESPACE),
        doctest.DocTestSuite('zope.ucol.localeadapter')
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Trivial adapter that adapts a zope.i18n locale to a collator

This adapter takes an object that has a getLocaleID method that
returns a locale string.  It returns a Collator for the given locale:

    >>> class Locale:
    ...     def __init__(self, id):
    ...         self.id = id
    ...     def getLocaleID(self):
    ...         return self.id

    >>> collator = LocaleCollator(Locale('da_DK'))
    >>> collator.__class__.__name__
    'Collator'

    >>> collator.locale
    'da_DK'

Note that we're not declaring any interfaces so as to avoid creating
a dependency on zope.i18n.locales.

$Id: localeadapter.py 67613 2006-04-26 08:21:40Z dobe $
"""
from zope.ucol import Collator

def LocaleCollator(locale):
    return Collator(str(locale.getLocaleID()))
    



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Simple wrapper for ICU ucol API

$Id$
"""
import sys

cdef extern from  "unicode/utypes.h":

    cdef enum UErrorCode:
        U_USING_DEFAULT_WARNING = -127
        U_USING_FALLBACK_WARNING = -128
    ctypedef int int32_t
    ctypedef char uint8_t
    int U_FAILURE(UErrorCode status)
    UErrorCode U_ZERO_ERROR

cdef extern from  "unicode/utf.h":

    ctypedef int UChar
    ctypedef int UChar32

cdef extern from  "unicode/ustring.h":
    
    UChar *u_strFromUTF32(UChar *dest, int32_t destCapacity,
                          int32_t *pDestLength,
                          UChar32 *src, int32_t srcLength,
                          UErrorCode *status)

cdef extern from  "unicode/ucol.h":

    ctypedef struct UCollator:
        pass
    UCollator *ucol_open(char *locale, UErrorCode *status)
    void ucol_close(UCollator *collator)
    int32_t ucol_getSortKey(UCollator *coll,
                            UChar *source, int32_t sourceLength,
                            uint8_t *result,
                            int32_t resultLength
                            )
    int ucol_strcoll(UCollator *coll,
                     UChar *source, int32_t sourceLength,
                     UChar *target, int32_t targetLength)

cdef extern from  "Python.h":

    int PyUnicode_Check(ob)
    int PyString_Check(ob)

    ctypedef int Py_UNICODE
    Py_UNICODE *PyUnicode_AS_UNICODE(ob)
    int PyUnicode_GET_SIZE(ob)
    char *PyString_AS_STRING(ob)

    void *PyMem_Malloc(int size)
    void PyMem_Free(void *p)
    object PyString_FromStringAndSize(char *v, int l)
    
    
cdef class UCharString:
    """Wrapper for ICU UChar arrays
    """

    cdef UChar *data
    cdef readonly int32_t length
    cdef readonly object base
    cdef readonly int need_to_free

    def __new__(self, text):
        cdef int32_t buffsize
        cdef UErrorCode status
        cdef Py_UNICODE *str
        cdef int length

        if not PyUnicode_Check(text):
            if PyString_Check(text):
                text = unicode(text)
                assert PyUnicode_Check(text)
            else:
                raise TypeError("Expected unicode string")

        length = PyUnicode_GET_SIZE(text)
        str = PyUnicode_AS_UNICODE(text)
        

        if sizeof(Py_UNICODE) == 2:
            self.data = str
            self.length = length
            self.base = text
            self.need_to_free = 0
        else:
            buffsize = 2*length + 1
            self.data = <UChar*>PyMem_Malloc(buffsize*sizeof(UChar))
            if self.data == NULL:
                raise MemoryError
            status = 0
            u_strFromUTF32(self.data, buffsize, &(self.length),
                           <UChar32*>str, length, &status)
            assert self.length <= buffsize
            self.need_to_free = 1
            if U_FAILURE(status):
                raise ValueError(
                    "Couldn't convert Python unicode data to ICU unicode data."
                    )

    def __dealloc__(self):
        if self.need_to_free and self.data != NULL:
            PyMem_Free(self.data)
            self.data = NULL


cdef class Collator:
    """Compute a collation key for a unicode string.
    """

    cdef UCollator *collator
    cdef readonly object locale
    cdef readonly int used_default_information

    def __new__(self, locale):
        cdef UCollator *collator
        cdef UErrorCode status

        if not PyString_Check(locale):
            raise TypeError("String locale expected")
        
        status = U_ZERO_ERROR
        collator = ucol_open(PyString_AS_STRING(locale), &status)
        if U_FAILURE(status):
            raise ValueError("Couldn't create a collator")
        self.collator = collator
        self.locale = locale
        if (status == U_USING_DEFAULT_WARNING
            or
            status == U_USING_FALLBACK_WARNING):
            status = 1
        self.used_default_information = status

    def __dealloc__(self):
        if self.collator != NULL:
            ucol_close(self.collator)

    def key(self, text):
        """Compute a collation key for the given unicode text.

        Of course, the key is only valid for the given locale.
        """
        cdef char *buffer
        cdef int32_t bufsize
        cdef int32_t size

        icutext = UCharString(text)
        bufsize = (<UCharString>icutext).length*2+10

        # the +1 below is needed to avoid an apprent buffer overflow bug in ICU
        buffer = <char*>PyMem_Malloc(bufsize +1)
        if buffer == NULL:
            raise MemoryError
        size = ucol_getSortKey(self.collator,
                               (<UCharString>icutext).data,
                               (<UCharString>icutext).length,
                               <uint8_t*>buffer, bufsize)
        while size > bufsize:
            bufsize = size
            PyMem_Free(buffer)
            buffer = <char*>PyMem_Malloc(bufsize +1) # See above +1
            if buffer == NULL:
                raise MemoryError
            size = ucol_getSortKey(self.collator,
                                   (<UCharString>icutext).data,
                                   (<UCharString>icutext).length,
                                   <uint8_t*>buffer, bufsize)

        result = PyString_FromStringAndSize(buffer, size)
        PyMem_Free(buffer)
        return result

    def cmp(self, o1, o2):
        u1 = UCharString(o1)
        u2 = UCharString(o2)
        return ucol_strcoll(
            self.collator,
            (<UCharString>u1).data,
            (<UCharString>u1).length,
            (<UCharString>u2).data,
            (<UCharString>u2).length,
            )



import os, sys, shutil, tempfile, urllib2
import setuptools.archive_util

class Recipe:

    def __init__(self, buildout, name, options):
        self.name = name
        self.options = options
        self.location = os.path.join(
            buildout['buildout']['parts-directory'],
            self.name)
        options['location'] = self.location
        if sys.platform.startswith('linux'):
            platform = 'LinuxRedHat'
        elif sys.platform.startswith('darwin'):
            platform = 'MacOSX'
        elif sys.platform.startswith('win32'):
            platform = 'win32'
        else:
            raise SystemError("Can't guess an ICU platform")
        options['platform'] = platform

    def install(self):
        options = self.options
        dest = options['location']
        if os.path.exists(dest):
            return dest

        if options['platform'] == 'win32':
            return self.install_win32(options, dest)
        
        here = os.getcwd()
        tmp = tempfile.mkdtemp()
        try:
            f = urllib2.urlopen(
                'ftp://ftp.software.ibm.com/software/globalization/icu/'
                '%(version)s/icu-%(version)s.tgz'
                % dict(version=options['version'])
                )
            open(os.path.join(tmp, 'arch'), 'w').write(f.read())
            f.close()
            setuptools.archive_util.unpack_archive(
                os.path.join(tmp, 'arch'),
                tmp,
                )
            os.chdir(os.path.join(tmp, 'icu', 'source'))
            assert os.spawnl(
                os.P_WAIT, 
                os.path.join(tmp, 'icu', 'source', 'runConfigureICU'),
                os.path.join(tmp, 'icu', 'source', 'runConfigureICU'),
                options['platform'],
                '--prefix='+dest,
                ) == 0
            assert os.spawnlp(os.P_WAIT, 'make', 'make', 'install') == 0
        finally:
            os.chdir(here)
            shutil.rmtree(tmp)

        return dest

    def update(self):
        pass

    def install_win32(self, options, dest):
        tmp = tempfile.mkstemp()
        try:
            f = urllib2.urlopen(
                'ftp://ftp.software.ibm.com/software/globalization/icu/'
                '%(version)s/icu-%(version)s-Win32-msvc7.1.zip'
                % dict(version=options['version'])
                )
            open(tmp, 'w').write(f.read())
            f.close()
            setuptools.archive_util.unpack_archive(tmp, dest)
        finally:
            shutil.rmfile(tmp)

        return dest
        



import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

name = "zc.recipe.icu"
setup(
    name = name,
    version = "1.0.0b1",
    author = "Jim Fulton",
    author_email = "jim@zope.com",
    description = ("ZC Buildout recipe for installing the ICU library"
                   " into a buildout"),
    long_description= read('README.txt'),
    license = "ZPL 2.1",
    keywords = "development build internationalization",
    url='http://www.python.org/pypi/'+name,

    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['zc', 'zc.recipe'],
    install_requires = ['setuptools'],
    entry_points = {'zc.buildout': ['default = %s:Recipe' % name]},
    classifiers = [
       'Framework :: Buildout',
       'Development Status :: 4 - Beta',
       'Intended Audience :: Developers',
       'License :: OSI Approved :: Zope Public License',
       ],
    )



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup for zope.ucol package

$Id: setup.py 70722 2006-10-16 17:20:28Z jim $
"""

import os, sys

from distutils.core import setup, Extension

try:
    from setuptools import setup, Extension
except:
    setuptools_options = {}
else:

    setuptools_options = dict(
        zip_safe = False,
        include_package_data = True,
        )

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

if sys.platform.startswith('win'):
    libraries = ['icuin', 'icuuc', 'icudt']
else:
    libraries=['icui18n', 'icuuc', 'icudata']

name = 'zope.ucol'
setup(name=name,
      version='1.0.2',
      author = "Jim Fulton",
      author_email = "jim@zope.com",
      description = "Python access to ICU text collation",
      long_description=(
        read('README.txt')
        + '\n' +
        read('INSTALL.txt')
        + '\n' +
        'Detailed Documentation\n'
        '**********************\n'
        + '\n' +
        read('src', 'zope', 'ucol', 'README.txt')
        + '\n' +
        'Download\n'
        '**********************\n'
        ),
      license = "ZPL 2.1",
      keywords = "internationalization",
      url = 'http://www.python.org/pypi/zope.ucol',
      ext_modules=[
          Extension('zope.ucol._zope_ucol',
                    ['src/zope/ucol/_zope_ucol.c'],
                    libraries=libraries,
           )
          ],
      packages=["zope", "zope.ucol"],
      package_dir = {'': 'src'},
      classifiers = [
       'Development Status :: 5 - Production/Stable',
       'Intended Audience :: Developers',
       'License :: OSI Approved :: Zope Public License',
       'Topic :: Software Development :: Libraries :: Python Modules',
       'Topic :: Software Development :: Internationalization',
       ],

      **setuptools_options)



#!/home/jim/p/z4i/jim-icu/var/opt/python/bin/python -u
##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Run tests in a Zope instance home.

$Id: test.py 40612 2005-12-06 20:52:16Z jim $
"""
import sys
sys.path.insert(0, 'src')
from zope.testing import testrunner

defaults = '--test-path src -szope.ucol'.split()

status = testrunner.run(defaults)

sys.exit(status)
